//***********************************************************************/
//*                                                                     */
//*     IF-70@API_Test  for Visual C#2008                              */
//*       Interface    USB / RS-232C                                    */
//*                                                                     */
//*     Copyright(c) 2014  TEXIO TECHNOLOGY CORPORATION                 */
//***********************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;           // for DllImport

namespace PDSA_Access
{
    public partial class Form1 : Form
    {
        // IF-60 static value
        private int pc_addr  = 1;                // PSA PC-ADDRESS
        private int sys_addr = 1;                // PSA SYSTEM-ADDRESS
        private int com_if   = -1;               // 0:USB  1:NI-GPIB  2:RS-232C  -1:Offline

        // DLL Import for IF-60
        [DllImport("if_60.dll")] static extern int USB488_DeviceOpen(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_DeviceClose(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_SetTimeout(int addrs, int m);
        [DllImport("if_60.dll")] static extern int USB488_SetRemote(int addrs, int m);
        [DllImport("if_60.dll")] static extern int USB488_GetRemote(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_DevClear(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_GetSTBQuery(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_ClearSTB(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_Send(int addrs, string p);
        [DllImport("if_60.dll")] static extern int USB488_Receive(int addrs, int cnt, StringBuilder p);

        // Form Init
        public Form1()
        {
            InitializeComponent();

            // Initialize the Combo1 with Interface
            Combo1.Items.Add("USB");
            Combo1.Items.Add("COM");

            Combo1.SelectedIndex = 0;

            // Initialize the ComboBox2 with the PC Address possible values
            for (int i = 1; i <= 30; i++)
            {
                Combo2.Items.Add(i);
            }
            Combo2.SelectedIndex = 0;

            // Initialize the ComboBox3 with the System Address possible values
            for (int i = 1; i <= 30; i++)
            {
                Combo3.Items.Add(i);
            }
            Combo3.SelectedIndex = 0;
            CommandEnableButton(0);
        }

        // Form Close (x Button)
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (Combo1.Enabled == false)
            {
                PSA_Send(com_if, pc_addr, "ADRS 0");              // Global Address
                PSA_Send(com_if, pc_addr, "OUTPUT 0");
                PSA_Send(com_if, pc_addr, "GTL"); 
                PSA_DeviceClose(com_if, pc_addr);
            }
        }
        
        // Click Open button 
        private void Open_Button_Click(object sender, EventArgs e)
        {
            // Get Addr&IF
            com_if = Combo1.SelectedIndex;
            pc_addr = Combo2.SelectedIndex + 1;
            sys_addr = Combo3.SelectedIndex + 1;

            if (Combo1.Enabled == true)
            {
                // Device Open
                if (0 < PSA_DeviceOpen(com_if, pc_addr))            //Open OK
                {
                    CommandEnableButton(1);
                    Text1.Text = "";
                }
                else
                {
                    CommandEnableButton(0);
                    Text1.Text = "Device Open Error";
                    com_if = -1;
                }

            }
            else
            {
                // Device Close
                CommandEnableButton(0);
                Text1.Text = "";
                PSA_Send(com_if, pc_addr, "ADRS 0");              // Global Address
                PSA_Send(com_if, pc_addr, "OUTPUT 0");
                PSA_Send(com_if, pc_addr, "GTL");
                PSA_DeviceClose(com_if, pc_addr);
                com_if = -1;
            }
        }

        // Enable Button
        private void CommandEnableButton(int mode)
        {
            Boolean sel;

            if (mode == 1)
            {
                sel = true;
                Open_Button.Text = "CLOSE";
            } else {
                sel = false;
                Open_Button.Text = "OPEN";
            }
            Output_Button.Text = "ON";
            Output_Button.BackColor = System.Drawing.SystemColors.Control;
            Output_Button.Enabled = sel;
            Pre1_Button.Enabled   = sel;
            Pre2_Button.Enabled   = sel;
            Pre3_Button.Enabled   = sel;
            Gtl_Button.Enabled    = sel;
            Read_Button.Enabled   = sel;
            
            if (mode == 0)
            {
                sel = true;
            } else {
                sel = false;
            }
            Combo1.Enabled  = sel;
            Combo2.Enabled  = sel;
            Combo3.Enabled  = sel;
        }

        //Click OUT Button
        private void Output_Button_Click(object sender, EventArgs e)
        {
            if (Output_Button.Text == "ON")
            {
                Output_Button.Text = "OFF";
                Output_Button.BackColor = System.Drawing.Color.Pink;
                PSA_Send(com_if, pc_addr, "OUTPUT 1");
            }
            else
            {
                Output_Button.Text = "ON";
                Output_Button.BackColor = System.Drawing.SystemColors.Control;
                PSA_Send(com_if, pc_addr, "OUTPUT 0\n");
            }
        }

        //Click Preset1 Button
        private void Pre1_Button_Click(object sender, EventArgs e)
        {
            PSA_Send(com_if, pc_addr, "PRESET 1");
        }

        //Click Preset2 Button
        private void Pre2_Button_Click(object sender, EventArgs e)
        {
            PSA_Send(com_if, pc_addr, "PRESET 2");
        }

        //Click Preset3 Button
        private void Pre3_Button_Click(object sender, EventArgs e)
        {
            PSA_Send(com_if, pc_addr, "PRESET 3");
        }

        //Click Preset OFF Button
        private void Gtl_Button_Click_1(object sender, EventArgs e)
        {
            PSA_Send(com_if, pc_addr, "GTL");

        }

        //Click Read Button        
        private void Read_Button_Click(object sender, EventArgs e)
        {
            string bbuf;
            string buf;
            double vol;
            double cur;
            double watt;

            PSA_Send(com_if, pc_addr, "STATUS?");
            bbuf = PSA_Read(com_if , pc_addr, 64);
            string[] mbuf = bbuf.Split(',');

            vol = double.Parse(mbuf[2]);
            cur = double.Parse(mbuf[3]);
            watt = vol * cur;
            Text1.Text = "READ DATA".PadLeft(17) + System.Environment.NewLine;
            buf = vol.ToString("F3");
            Text1.Text = Text1.Text + buf.PadLeft(17) + " [V]" + System.Environment.NewLine;
            buf = cur.ToString("F3");
            Text1.Text = Text1.Text + buf.PadLeft(17) + " [A]" + System.Environment.NewLine;
            buf = watt.ToString("F3");
            Text1.Text = Text1.Text + buf.PadLeft(17) + " [W]" + System.Environment.NewLine;
            Text1.Text = Text1.Text + mbuf[1].PadLeft(7) + " [0:CV/1:CC/2:OFF]";
        }

        // Display Change
        private void Combo1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Combo1.Text == "COM")
            {
                Label4.Text = "COM Port";
            }
            else
            {
                Label4.Text = "PC ADDR";
            }
        }

        // IF-60 Device Open        
        private int PSA_DeviceOpen(int comif, int  addr)
        {
            int hHandle;
            int retn;
            byte adrs;

            adrs = (byte)addr;
            hHandle = -1;
            switch (comif)
            {
                case 0: //USB
                    try
                    {
                        hHandle = USB488_DeviceOpen(addr);
                    }
                    catch (System.Exception caught)
                    {
                        MessageBox.Show(caught.Message, "Open Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        break; 
                    }

                    retn = USB488_SetTimeout(addr, 3000);
                    break;

                case 1: //RS-232C
                    try
                    {
                        serialPort1.PortName = "COM" + pc_addr.ToString();
                        serialPort1.BaudRate = 9600;
                        serialPort1.DataBits = 7;
                        serialPort1.Parity = System.IO.Ports.Parity.Even;
                        serialPort1.StopBits = System.IO.Ports.StopBits.One;
                        serialPort1.Handshake = System.IO.Ports.Handshake.None;
                        serialPort1.RtsEnable = true;
                        serialPort1.ReadTimeout = 3000;
                        serialPort1.WriteTimeout = 500;
                        serialPort1.Open();
                        serialPort1.DiscardInBuffer();
                        hHandle = pc_addr;
                    }
                    catch
                    {
                        hHandle = -1;
                    }
                        
                    break;

                default:
                    hHandle = -1;
                    break;
            }
            if (hHandle > 0)
            {
                retn = PSA_Send(comif, addr, "ADRS " + sys_addr.ToString());
                if (retn >= 0)
                {
                    retn = PSA_Send(comif, addr, "OUTPUT 0");
                    retn = PSA_Send(comif, addr, "MONDSP 0");
                }
                else
                {
                    hHandle = -1;
                    PSA_DeviceClose(comif, addr);
                }
            }
            return hHandle;
        }

        // IF-60 Device Close
        private int PSA_DeviceClose(int comif, int addr)
        {
            int retn;

            switch (comif)
            {
                case 0: //USB
                    retn = USB488_DeviceClose(addr);
                    break;

                case 1: //RS-232C
                    serialPort1.Close();
                    retn = 0;
                    break;

                default:
                    retn = -1;
                    break;
            }
            return retn;
        }

        // IF-60 Device Send Msg
        private int PSA_Send(int comif, int addr, string msg)
        {
            int retn;

            if ((msg.Length > 256)|| (msg.Length == 0)) return -2;

            retn = -1;
            try
            {
                switch (comif)
                {
                    case 0: //USB
                        retn = USB488_Send(addr, msg );
                        break;
                    
                    case 1: //RS-232C
                        serialPort1.DiscardInBuffer();
                        serialPort1.WriteLine(msg + "\n");
                        retn = 0;
                        break;

                    default:
                        break;
                }
            }
            catch (System.Exception caught)
            {
                MessageBox.Show(caught.Message, "SendErr", MessageBoxButtons.OK, MessageBoxIcon.Error);
                retn = -1;
            }
            return retn;
        }

        // IF-60 Device Read Msg
        private string PSA_Read(int comif, int addr, int cnt)
        {
            int lStatus = -1;
            string tmpstr = "";
            StringBuilder rstr = new StringBuilder(256);

            try
            {
                switch (comif)
                {
                    case 0: //USB
                        lStatus = USB488_Receive(addr, cnt, rstr );
                        tmpstr = rstr.ToString();
                        break;

                    case 1: //RS-232C
                        System.Threading.Thread.Sleep(200);
                        tmpstr = serialPort1.ReadExisting() ;
                        lStatus = tmpstr.Length;
                        break;

                    default :
                        break;
                }
                if (lStatus <= 1)
                {
                    MessageBox.Show("Read Error", "Read Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    tmpstr = "";
                }
            }
            catch (System.Exception caught)
            {
                MessageBox.Show(caught.Message, "Read Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                tmpstr = "";
            }
            return tmpstr;
        }


    }
}